<?php
// $Horde: turba/lib/Source.php,v 1.30 2001/05/31 20:53:41 chuck Exp $

require_once TURBA_BASE . '/lib/Turba.php';

/**
 * The Turba_Source:: class provides a set of methods for dealing with
 * specific contact sources.
 *
 * @author  Chuck Hagenbuch <chuck@horde.org>
 * @author  Jon Parise <jon@csh.rit.edu>
 * @version $Revision: 1.30 $
 * @since   Turba 0.0.1
 * @package turba
 */
class Turba_Source {
    
    /** String containing the internal name of this source. */
    var $name = '';
    
    /** String containing the symbolic title of this source. */
    var $title;
    
    /** Instance of the underlying Turba_Driver class. */
    var $driver;
    
    /** Hash describing the mapping between Turba attributes and
        driver-specific fields. */
    var $map = array();
    
    /** Boolean indicating whether this source is publicly searchable. */
    var $public = false;
    
    /** Boolean indicating whether this source is read-only (not editable). */
    var $readonly = true;
    
    /** Object holding any PEAR_Errors raised inside the constructor. */
    var $error = null;
    
    
    /**
     * Constructs a new Turba_Source object.
     *
     * @param $name         String containing the internal name of this source.
     * @param $title        String containing the symbolic title of this source.
     * @param $driver       String indicating the underlying Turba_Driver
     *                      subclass.
     * @param $params       Additional parameters to pass to the driver
     *                      instance.
     * @param $map          Mapping between Turba attributes and
     *                      driver-specific fields.
     * @param $public       (optional) Boolean indicating whether this is a
     *                      public source.
     * @param $readonly     (optional) Boolean indicating whether this is a
     *                      read-only (non editable) source.
     */
    function Turba_Source($name, $title, $driver, $params, $map,
                          $public = false, $readonly = true)
    {
        $this->name = $name;
        $this->title = $title;
        
        /*
         * Obtain a handle to a driver of the requested type.  If an
         * instance of that driver doesn't already exist, a new one will be
         * created and returned.
         */
        require_once TURBA_BASE . '/lib/Driver.php';
        $this->driver = &Turba_Driver::singleton($driver, $params);
        if (!empty($this->driver->errno)) {
            $this->error = new PEAR_Error('Driver instantiation failed');
            return false;
        }
        
        /* Store and translate the map at the Source level. */
        $this->map = $map;
        
        /* Set flags. */
        $this->public = $public;
        $this->readonly = $readonly;
    }
    
    
    /**
     * Returns a formatted error message from the driver level.
     *
     * @return              String containing the formatted error message.
     */
    function error()
    {
        return $this->driver->error();
    }
    
    /**
     * Translates the keys of the first hash from the generalized Turba
     * attributes to the driver-specific fields.  The translation is based
     * on the contents of $this->map.
     *
     * @param $hash         Hash using Turba keys.
     *
     * @return              Translated version of $hash.
     */
    function toDriverKeys($hash)
    {
        $fields = array();
        foreach ($hash as $turba_key => $val) {
            if (isset($this->map[$turba_key])) {
                $driver_key = $this->map[$turba_key];
                $fields[$driver_key] = $val;
            }
        }
        return $fields;
    }
    
    /**
     * Translates the keys of the first hash from the driver-specific
     * fields to the generalized Turba attributes.  The translation is based
     * on the contents of $this->map.
     *
     * @param $hash         Hash using driver-specific keys.
     *
     * @return              Translated version of $hash.
     */
    function toTurbaKeys($hash)
    {
        $attributes = array();
        for ($i = 0; $i < count($hash); $i++) {
            $entry = $hash[$i];
            $new_entry = array();
            
            reset($this->map);
            foreach ($this->map as $turba_key => $val) {
                $new_entry[$turba_key] = $entry[$val];
            }
            
            $attributes[] = $new_entry;
        }
        return $attributes;
    }
    
    /**
     * Returns a hash of this object's current state for serialization.
     *
     * @return              Hash of this object's current state.
     */
    function pickle()
    {
        $state = array();
        $state['name'] = $this->name;
        $state['title'] = $this->title;
        $state['driver'] = $this->driver->getType();
        $state['params'] = $this->driver->getParams();
        $state['map'] = $this->map;
        $state['public'] = $this->public;
        $state['readonly'] = $this->readonly;
        
        return $state;
    }
    
    /**
     * Recreates a pickled instance of a Source object.
     * 
     * @param $state        The pickled state of the object.
     *
     * @return              The reinstated object.
     */
    function unpickle($state)
    {
        if (!is_array($state)) return false;

        return new Turba_Source($state['name'],
                                $state['title'],
                                $state['driver'],
                                $state['params'],
                                $state['map'],
                                $state['public'],
                                $state['readonly']);
    }
    
    /**
     * Searches the source based on the provided criteria.
     *
     * TODO: Allow $criteria to contain the comparison operator (<, =, >,
     *       'like') and modify the drivers accordingly.
     *
     * @param $search_criteria   Hash containing the search criteria.
     * @param $sort_criteria     The requested sort order which is passed to
     *                           Turba_List::sort().
     *
     * @return                   The sorted, filtered list of search results.
     */
    function search($search_criteria, $sort_criteria = LASTNAME)
    {
        include_once TURBA_BASE . '/lib/List.php';
        include_once TURBA_BASE . '/lib/Object.php';
        
        /*
         * If this is not a public source, enforce the requirement that the
         * source's owner must be equal to the current user.
         */
        $strict_fields = array();
        if (!$this->public) {
            $search_criteria['__owner'] = Auth::getAuth();
            $strict_fields = array_keys($this->toDriverKeys(array('__owner' => '')));
        }
        
        /* Translate the Turba attributes to the driver-specific fields. */
        $fields = $this->toDriverKeys($search_criteria);
        
        /* Retrieve the search results from the driver. */
        $results = $this->driver->search($fields, array_values($this->map), $strict_fields);
        
        /*
         * Translate the driver-specific fields in the result back to the 
         * more generalized common Turba attributes using the map.
         */
        $remapped_results = $this->toTurbaKeys($results);
        
        $list = new Turba_List();
        foreach ($remapped_results as $attributes) {
            $list->insert(new Turba_Object($this, $attributes));
        }
        $list->sort($sort_criteria);
        
        /* Return the filtered (sorted) results. */
        return $list;
    }
    
    /**
     * Retrieve one object from the source.
     *
     * @param string $objectID         The unique id of the object to retrieve.
     *
     * @return Turba_AbstractObject   The retrieved object.
     */
    function getObject($objectID)
    {
        $criteria = $this->map['__key'];
        
        $objects = $this->driver->read($criteria, $objectID, array_values($this->map));
        if (!is_array($objects) || count($objects) != 1) {
            return false;
        } else {
            $remapped_objects = $this->toTurbaKeys($objects);
            
            include_once TURBA_BASE . '/lib/Object.php';
            return new Turba_Object($this, $remapped_objects[0]);
        }
    }
    
    /**
     * Adds a new entry to the contact source.
     *
     * @param array $attributes     The attributes of the new object to add.
     */
    function addObject($attributes)
    {
        if ($this->readonly) return false;
        
        if (!isset($attributes['__key'])) {
            $attributes['__key'] = $this->driver->makeKey($this->toDriverKeys($attributes));
        }
        
        $key = $attributes['__key'];
        $attributes = $this->toDriverKeys($attributes);
        
        if ($this->driver->addObject($attributes)) {
            return $key;
        } else {
            return false;
        }
    }
    
    /**
     * Deletes the specified entry from the contact source.
     *
     * @param string $object_id     The ID of the object to delete.
     */
    function removeObject($object_id)
    {
        if ($this->readonly) return false;
        
        list($object_key,) = each($this->toDriverKeys(array('__key' => '')));
        return $this->driver->removeObject($object_key, $object_id);
    }
    
    /**
     * Modifies an existing entry in the contact source.
     *
     * @param Turba_AbstractObject $object     The object to update.
     */
    function setObject($object)
    {
        if ($this->readonly) return false;
        
        $attributes = $this->toDriverKeys($object->getAttributes());
        list($object_key, $object_id) = each($this->toDriverKeys(array('__key' => $object->getValue('__key'))));
    
        return $this->driver->setObject($object_key, $object_id, $attributes);
    }
    
    /**
     * Returns the criteria available for this source except '__key'.
     *
     * @return              An array containing the criteria.
     */
    function getCriteria()
    {
        $criteria = array();
        foreach ($this->map as $key => $value) {
            if ($key != '__key') {
                $criteria[$key] = $value;
            }
        }
        return $criteria;
    }
    
}
?>
